import pkg_resources

from typing import Any, Callable, List, Iterable

from py_hiverunner.core.type import JString, Deserialized


def array_cast(array: Iterable[JString], cast: Callable[[JString], Deserialized]) -> List[Deserialized]:
    _list = []
    for item in array:
        _list.append(cast(item))
    return _list


def list_string(string_array: str) -> List[Any]:
    if string_array == "[]":
        return []
    return [item[1:-1] for item in string_array.strip("][").split(",")]


def boolean_string(string_bool: str) -> bool:
    return string_bool == "true"


def default_classpath() -> str:
    return pkg_resources.resource_filename(__name__, "bin/hiverunner.jar")



import os

from dotenv import load_dotenv

from py_hiverunner.utils import default_classpath

load_dotenv()

HIVERUNNER_CLASS_PATH = os.path.abspath(os.environ.get("HIVERUNNER", default_classpath()))
BASEDIR = os.environ.get("PYHIVERUNNER_BASEDIR", "py-hiverunner-")
DEFAULT_SEP = "\t"
DEFAULT_NULL = "NULL"



from typing import List, TypeVar, Iterable, Union


JClass = TypeVar("JClass")
JHiveRunner = TypeVar("JHiveRunner")
HQL = str
JString = TypeVar("JString")
SourceArray = Iterable[int]
Deserialized = Union[int, float, List[str], str, bool]



__version__ = '0.1.2'

from py_hiverunner.jvm import Py4JEntryPoint
from py_hiverunner.parser import RegexParser
from py_hiverunner.hiverunner_api import HiveRunnerApi
from py_hiverunner.hiverunner import HiveRunner
from py_hiverunner.settings import BASEDIR, DEFAULT_SEP, DEFAULT_NULL


def hiverunner(**kwargs) -> HiveRunner:
    """
    Creates hiverunner instance.

    :param kwargs: Optional key arguments used by JVMEntryPoint
    :return: hiverunner
    :rtype: HiveRunner
    """
    _jvm = Py4JEntryPoint(**kwargs)
    _api = HiveRunnerApi(jvm=_jvm, basedir=BASEDIR)
    _parser = RegexParser(sep=DEFAULT_SEP, null_presentation=DEFAULT_NULL)
    return HiveRunner(_api, _parser)



import jpype

from typing import Optional

from py4j.java_gateway import JavaGateway, GatewayParameters

from py_hiverunner.core.jvm import JVMEntryPoint
from py_hiverunner.core.type import JHiveRunner, JClass


class JPypeEntryPoint(JVMEntryPoint):
    def __init__(self, classpath: str, import_path: str = "com.la9ran9e.hiverunner.HiveRunner"):
        self._classpath = classpath
        self._import_path = import_path
        self._class: Optional[JClass] = None

    def start(self):
        jpype.addClassPath(self._classpath)
        jpype.startJVM(convertStrings=False)
        self._class = jpype.JClass(self._import_path)

    def stop(self):
        jpype.shutdownJVM()

    def get_hiverunner(self, *args, **kwargs) -> JHiveRunner:
        return self._class(*args, **kwargs)


class Py4JEntryPoint(JVMEntryPoint):
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self._gateway: Optional[JavaGateway] = None
        self._instance = None

    def start(self):
        self._gateway = JavaGateway(gateway_parameters=GatewayParameters(**self._kwargs))

    def stop(self):
        if self._instance:
            self._instance.stop()
        self._gateway.close()

    def get_hiverunner(self, *args, **kwargs) -> JHiveRunner:
        if self._instance:
            self._instance.stop()
        self._instance = self._gateway.entry_point.getHiveRunnerInstance(*args, **kwargs)
        return self._instance



from typing import List, Optional, Tuple

from py_hiverunner.hiverunner_api import HiveRunnerApi
from py_hiverunner.core.parser import Parser
from py_hiverunner.core.type import HQL, Deserialized


class HiveRunner:
    def __init__(self, api: HiveRunnerApi, parser: Parser):
        self._api = api
        self._parser = parser

    def execute_query(self, query: HQL) -> List[Tuple[Optional[Deserialized], ...]]:
        raw_result = self._api.execute_query(query)
        result = []
        for string_row in raw_result:
            result.append(self._parser.parse_string(string_row))
        return result

    def start(self):
        self._api.start()

    def stop(self):
        self._api.stop()

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()



import re
from typing import Any, Callable, Optional, Tuple

from py_hiverunner.core.parser import Parser
from py_hiverunner.utils import list_string, boolean_string
from py_hiverunner.core.type import Deserialized


class RegexParser(Parser):
    TYPE_EXPRESSIONS = [
        r"(?P<list>^\[(\"[^\"\\]*(?:\\.[^\"\\]*)*\",*)*\]$)",   # list
        r"(?P<float>^-*\d*\.\d+$)",                             # float
        r"(?P<int>^-*\d+$)",                                    # int
        r"(?P<boolean>^(true|false)$)",                         # boolean
    ]
    _regex = re.compile("|".join(TYPE_EXPRESSIONS))
    _types = {
        "list": list_string,
        "int": int,
        "float": float,
        "boolean": boolean_string,
    }

    def __init__(self, sep: str = "\t", null_presentation: str = "NULL"):
        self._sep = sep
        self._null_presentation = null_presentation

    def parse_string(self, string: str) -> Tuple[Optional[Deserialized], ...]:
        parsed = []
        parts = string.split(self._sep)
        for item in parts:
            if item == self._null_presentation:
                parsed_item = None
            else:
                cast = self._typeof(item)
                parsed_item = cast(item)
            parsed.append(parsed_item)
        return tuple(parsed)

    def _typeof(self, string: str) -> Callable[[Any], Any]:
        res = self._regex.search(string)
        if res:
            return self._types[res.lastgroup]
        return str



from abc import ABC, abstractmethod
from typing import Optional, Tuple

from py_hiverunner.core.type import Deserialized


class Parser(ABC):
    @abstractmethod
    def parse_string(self, string: str) -> Tuple[Optional[Deserialized], ...]:
        ...



from abc import ABC, abstractmethod

from py_hiverunner.core.type import JHiveRunner


class JVMEntryPoint(ABC):
    @abstractmethod
    def start(self):
        ...

    @abstractmethod
    def stop(self):
        ...

    @abstractmethod
    def get_hiverunner(self, *args, **kwargs) -> JHiveRunner:
        ...



from typing import List

from py_hiverunner.core.jvm import JVMEntryPoint
from py_hiverunner.core.type import HQL
from py_hiverunner.utils import array_cast


class HiveRunnerApi:
    def __init__(self, jvm: JVMEntryPoint, basedir: str = "py-hiverunner-"):
        self._jvm = jvm
        self._basedir = basedir
        self._instance = None

    def start(self):
        self._jvm.start()
        self._instance = self._jvm.get_hiverunner(self._basedir)
        self._instance.setup()

    def stop(self):
        self._instance.stop()
        self._jvm.stop()

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()

    def execute_query(self, query: HQL) -> List[str]:
        ret = self._instance.executeQuery(query)
        return array_cast(ret, str)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['py_hiverunner', 'py_hiverunner.core']

package_data = \
{'': ['*']}

install_requires = \
['JPype1>=0.7.5,<0.8.0',
 'py4j>=0.10.9,<0.11.0',
 'python-dotenv>=0.13.0,<0.14.0']

setup_kwargs = {
    'name': 'py-hiverunner',
    'version': '0.1.2',
    'description': 'Python API for unittest Hive applications',
    'long_description': '# py-hiverunner\n[![Build Status](https://travis-ci.com/la9ran9e/py-hiverunner.svg?branch=master)](https://travis-ci.com/la9ran9e/py-hiverunner)\n[![Code Coverage Status](https://codecov.io/gh/la9ran9e/py-hiverunner/branch/master/graph/badge.svg)](https://codecov.io/gh/la9ran9e/py-hiverunner)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/py-hiverunner)](https://pypi.org/project/py-hiverunner/)\n[![Docker Build Status](https://img.shields.io/docker/cloud/build/la9ran9e/py-hiverunner)](https://hub.docker.com/r/la9ran9e/py-hiverunner)\n\nLibrary provides python API for [Klarna\'s HiveRunner](https://github.com/klarna/HiveRunner).\n\n## Install\n\nInstall `py-hiverunner` package with [pip](https://pypi.org/project/py-hiverunner/):\n\n```bash\n$ python -m pip install py-hiverunner\n```\n\n## Usage\nBefore using `py-hiverunner` you need start JVM with facade service based on original Klarna\'s `HiveRunner` - \n[java-hiverunner](./java-hiverunner).\nThis repo provides [docker container for Py4J Java Gateway for HiveRunner](https://hub.docker.com/r/la9ran9e/py-hiverunner).\n\nYou can pull this:\n```bash\n$ docker pull la9ran9e/py-hiverunner\n```\nand then run the container:\n```bash\n$ docker run -ti -p 25333:25333 -p 25334:25334 la9ran9e/py-hiverunner\n```\nAfter that you will have working server with Java HiveRunner.\n\nTry this:\n\n```python\nfrom py_hiverunner import hiverunner\nfrom pprint import pprint\n\n\nwith hiverunner() as hive:\n    hive.execute_query("create schema meh")\n    hive.execute_query("create table meh.nonsub(a int, b string, c array<string>)")\n    hive.execute_query("insert into meh.nonsub select 1, \'la9ran9e\', array(\'1\', \'a\', \'b\', \'6\')")\n    hive.execute_query("insert into meh.nonsub select 2, \'la9ran9e\', array(\'1\', \'b\', \'b\', \'6\')")\n    hive.execute_query("insert into meh.nonsub select 3, \'la9ran9e\', array(\'1\', \'c\', \'b\', \'6\')")\n    hive.execute_query("insert into meh.nonsub select 4, \'\', array(\'1\', \'d\', \'b\', \'6\')")\n\n    hive.execute_query("create table meh.sub(a int, b string, c boolean)")\n    hive.execute_query("insert into meh.sub select 1, \'la9ran9e\', true")\n\n    print("RESULT:")\n    pprint(hive.execute_query("""\n        select\n            *\n        from\n            meh.sub as sub\n        inner join\n            meh.nonsub as nonsub\n        on\n            sub.b = nonsub.b\n    """))\n```',
    'author': 'la9ran9e',
    'author_email': 'tvauritimur@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/la9ran9e/py-hiverunner',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
